<?php
//
// LMS-API version 0,1,0
//
// Copyright 2002 DuckCorp and RtpNet
//
// This program is free software ; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation (http://www.gnu.org/).
//



/**
* LDAP Connection Handling Class
*
* @access	public
*/
class LmsLdapConnection extends LmsRoot
{
	/**
	* Connected to an LDAP server ?
	*
	* @access	private
	* @var		boolean
	*/
	var	$_connected;
	/**
	* LDAP Server
	*
	* @access	private
	* @var		string
	*/
	var	$_server;
	/**
	* LDAP Ressource
	*
	* @access	private
	* @var		integer
	*/
	var	$_ds;
	/**
	* Logged in ?
	*
	* @access	private
	* @var		boolean
	*/
	var	$_binded;
	/**
	* User Bind DN
	*
	* @access	private
	* @var		string
	*/
	var	$_bind_dn;
	/**
	* User Bind Password
	*
	* @access	private
	* @var		string
	*/
	var	$_bind_passwd;
	/**
	* User Type
	*
	* 0 => Unknown User Type
	* 1 => Common User
	* 2 => Admin User
	* 3 => Service User
	*
	* @access	private
	* @var		integer
	*/
	var	$_user_type;
	/**
	* Helping parameters set ?
	*
	* @access	private
	* @var		boolean
	*/
	var	$_has_wrapping_parameters;
	/**
	* LDAP Base DN
	*
	* @access	private
	* @var		string
	*/
	var	$_base_dn;
	/**
	* User part DN
	*
	* @access	private
	* @var		string
	*/
	var	$_user_part_dn;
	/**
	* Common User part DN
	*
	* @access	private
	* @var		string
	*/
	var	$_commonuser_part_dn;
	/**
	* Admin part User
	*
	* @access	private
	* @var		string
	*/
	var	$_adminuser_part_dn;
	/**
	* Service User part DN
	*
	* @access	private
	* @var		string
	*/
	var	$_serviceuser_part_dn;

	function LmsLdapConnection()
	{
		LmsRoot::LmsRoot();
		
		$this->_init();
		
		register_shutdown_function(array(&$this, "_LmsLdapConnection"));
	}

	function _LmsLdapConnection()
	{
		if ($this->_connected)
			$this->disconnect();
	}

	/**
	* Initialisation
	*
	* @access	private
	*/
	function _init()
	{
		$this->_server = "";
		$this->_connected = false;
		$this->_ds = 0;
		$this->_binded = false;
		$this->_bind_dn = "";
		$this->_bind_passwd = "";
		$this->_user_type = 0;
		$this->_has_wrapping_parameters = "";
		$this->_base_dn = "";
		$this->_user_part_dn = "";
		$this->_commonuser_part_dn = "";
		$this->_adminuser_part_dn = "";
		$this->_serviceuser_part_dn = "";
	}

	/**
	* Connect to an LDAP server
	*
	* @access	public
	* @param	string		$server		Server Name
	* @param	boolean		$checkv3	Protocol version 3 needed ?
	* @return	boolean		$ok		True if successful
	*/
	function connect($server, $checkv3 = true)
	{
		$this->_server = $server;
		
		if ($this->_connected)
			$this->disconnect();

		$this->_ds = ldap_connect($this->_server);
		if ($this->_ds)
		{
			if ($checkv3 && !ldap_set_option($this->_ds, LDAP_OPT_PROTOCOL_VERSION, 3))
			{
				$this->disconnect();
				$this->_set_error("Failed to set protocol version to 3");
			}
			else
			{
				$this->_connected = true;
				$this->_set_error();
				return true;
			}
		}
		else
			$this->_set_error("Cannot find LDAP Server");

		return false;
	}

	/**
	* Disconnect from an LDAP server
	*
	* @access	public
	* @return	boolean		$ok		True if successful
	*/
	function disconnect()
	{
		if (!$this->_prerequisite(false, false))
			return false;

		ldap_close($this->_ds);
		$this->_init();
		return true;
	}

	/**
	* log into an ldap server
	*
	* @access	public
	* @param	string		$bind_dn	dn of the user
	* @param	string		$bind_passwd	password for the user
	* @return	boolean		$ok		true if successful
	*/
	function bind($bind_dn, $bind_passwd)
	{
		if (!$this->_prerequisite(false, false))
			return false;

		if (ldap_bind($this->_ds, $bind_dn, $bind_passwd))
		{
			$this->_binded =true;
			$this->_bind_dn = $bind_dn;
			$this->_bind_passwd = $bind_passwd;
			$this->_user_type = 0;
			
			$this->_set_error();
			return true;
		}

		$this->_set_error("Cannot Bind as '".$bind_dn."'");
		return false;
	}

	/**
	* Set helping parameters
	*
	* @access	public
	* @param	string		$base_dn	DN of the root node of LDAP
	* @param	string		$user_part_dn	Part of the DN to add to find all the users
	* @param	string		$commonuser_part_dn	Part of the DN to add to find the common users
	* @param	string		$adminuser_part_dn	Part of the DN to add to find the admin users
	* @param	string		$serviceuser_part_dn	Part of the DN to add to find the service users
	* @return	boolean		$ok		True if successful
	*/
	function set_wrapping_parameter($base_dn, $user_part_dn, $commonuser_part_dn, $adminuser_part_dn, $serviceuser_part_dn)
	{
		$this->_has_wrapping_parameters = true;
		$this->_base_dn = $base_dn;
		$this->_user_part_dn = $user_part_dn;
		$this->_commonuser_part_dn = $commonuser_part_dn;
		$this->_adminuser_part_dn = $adminuser_part_dn;
		$this->_serviceuser_part_dn = $serviceuser_part_dn;

		$this->_set_error();
		return true;
	}
	
	/**
	* log into an ldap server
	*
	* @access	public
	* @param	string		$bind_user	UID for the user
	* @param	string		$bind_passwd	password for the user
	* @return	boolean		$ok		true if successful
	*/
	function w_bind($bind_user, $bind_passwd)
	{
		if (!$this->_prerequisite(false, true))
			return false;

		$bind_dn = "uid=".$bind_user.",".$this->_commonuser_part_dn.",".$this->_user_part_dn.",".$this->_base_dn;
		if (!ldap_bind($this->_ds, $bind_dn, $bind_passwd))
		{
			$bind_dn = "uid=".$bind_user.",".$this->_adminuser_part_dn.",".$this->_user_part_dn.",".$this->_base_dn;
			if (!ldap_bind($this->_ds, $bind_dn, $bind_passwd))
			{
				$bind_dn = "uid=".$bind_user.",".$this->_serviceuser_part_dn.",".$this->_user_part_dn.",".$this->_base_dn;
				if (!ldap_bind($this->_ds, $bind_dn, $bind_passwd))
				{
					$this->_set_error("Cannot Bind as '".$bind_user."'");
					return false;
				}
				else
					$this->_user_type = 3;
			}
			else
				$this->_user_type = 2;
		}
		else
			$this->_user_type = 1;
		
		$this->_binded =true;
		$this->_bind_dn = $bind_dn;
		$this->_bind_passwd = $bind_passwd;
			
		$this->_set_error();
		return true;
	}

	/**
	* Give the user type (Admin/Common/Service)
	*
	* @access	public
	* @return	integer
	*/
	function get_user_type()
	{
		if (!$this->_prerequisite(true, false))
			return false;

		return $this->_user_type;
	}

	/**
	* Search in the LDAP Database
	*
	* @access	public
	* @param	string		$base		DN where to begin the search
	* @param	string		$search		Search pattern
	* @param	object LmsLdapEntries	&$entries	Object where to put results
	* @param	array		$attr		Array of attribut keys to fetch
	* @param	boolean		$subtree	Search in subtrees ?
	* @return	boolean		$ok		true if successful
	*/
	function search($base, $search, &$entries, $attr = array(), $subtree = true)
	{
		if (!$this->_prerequisite(true, false))
			return false;

		if ($subtree)
			$f = "ldap_search";
		else
			$f = "ldap_list";
			
		if (is_array($attr) && count($attr)>0)
			$sr = $f($this->_ds, $base, $search, $attr);
		else
			$sr = $f($this->_ds, $base, $search);

		if ($sr)
		{
			$nb = ldap_count_entries($this->_ds, $sr);
			if (!$nb)
				$tab = array();
			else
				$tab = ldap_get_entries($this->_ds, $sr);

			ldap_free_result($sr);
			$entries->_put_tab($tab, $nb);

			return true;
		}
		
		$this->_set_error("LDAP Error : ".ldap_error($this->_ds));
		return false;
	}

	/**
	* Search in all the LDAP Database
	*
	* @access	public
	* @param	string		$search		Search pattern
	* @param	object LmsLdapEntries	&$entries	Object where to put results
	* @param	array		$attr		Array of attribut keys to fetch
	* @param	boolean		$subtree	Search in subtrees ?
	* @return	boolean		$ok		true if successful
	*/
	function w_search($search, &$entries, $attr = array(), $subtree = true)
	{
		if (!$this->_prerequisite(true, true))
			return false;

		return $this->search($this->_base_dn, $search, $entries, $attr);
	}

	/**
	* Modify an LDAP Object
	*
	* @access	public
	* @param	string		$object		DN of the Object
	* @param	object LmsLdapEntries	&$entries	Object data
	* @param	integer		$i		Number of the entry to Replace
	* @param	array		$attr_list	List of Attributes to Replace
	* @return	boolean		$ok		true if successful
	*/
	function modify($object, &$entries, $i, $attr_list = array())
	{
		if (!$this->_prerequisite(true, false))
			return false;

		if (count($attr_list)!=0)
		{
			if (ldap_mod_replace($this->_ds, $object, $entries->_export_entry($i, $attr_list)))
				return true;
		}
		else
		{
			if (ldap_modify($this->_ds, $object, $entries->_export_entry($i)))
				return true;
		}

		$this->_set_error("LDAP Error : ".ldap_error($this->_ds));
		return false;
	}

	/**
	* Add an LDAP Object
	*
	* @access	public
	* @param	string		$object		DN of the Object
	* @param	object LmsLdapEntries	&$entries	Object data
	* @param	integer		$i		Number of the entry to Replace
	* @return	boolean		$ok		true if successful
	*/
	function add($object, &$entries, $i)
	{
		if (!$this->_prerequisite(true, false))
			return false;

		if (ldap_add($this->_ds, $object, $entries->_export_entry($i)))
			return true;

		$this->_set_error("LDAP Error : ".ldap_error($this->_ds));
		return false;
	}

	/**
	* Delete an LDAP Object
	*
	* @access	public
	* @param	string		$object		DN of the Object
	* @return	boolean		$ok		true if successful
	*/
	function delete($object)
	{
		if (!$this->_prerequisite(true, false))
			return false;;
		if (!$object)
			return false;

		if (ldap_delete($this->_ds, $object))
			return true;

		$this->_set_error("LDAP Error : ".ldap_error($this->_ds));
		return false;
	}

	/**
	* Rename an LDAP Object
	*
	* @access	public
	* @param	string		$object		DN of the Object
	* @param	string		$new_object	New DN of the Object
	* @return	boolean		$ok		true if successful
	*/
	function rename($object, $new_object)
	{
		if (!$this->_prerequisite(true, false))
			return false;

		$z = ldap_explode_dn($new_object, 0);
		$newrdn = $z[0];
		array_splice($z, 0, 2);
		$newparent = implode(",", $z);
		if (ldap_rename($this->_ds, $object, $newrdn, $newparent, false))
			return true;

		$this->_set_error("LDAP Error : ".ldap_error($this->_ds));
		return false;
	}

	/**
	* Check prerequisite for other methodes
	*
	* @access	private
	* @param	boolean		$need_bind	Is login needed ?
	* @param	boolean		$need_wrapping_parameters	Are helping parameters needed ?
	* @return	boolean		$ok		true if successful
	*/
	function _prerequisite($need_bind, $need_wrapping_parameters)
	{
		if (!$this->_connected)
		{
			$this->_set_error("Not connected to any server");
			return false;
		}
		if ($need_bind && !$this->_binded)
		{
			$this->_set_error("Not binded to any account");
			return false;
		}
		if ($need_wrapping_parameters && !$this->_has_wrapping_parameters)
		{
			$this->_set_error("Wrapping parameters not set");
			return false;
		}

		return true;
	}
}

?>
